Package: hdnom
Type: Package
Title: Benchmarking and Visualization Toolkit for Penalized Cox Models
Version: 6.2.1
Authors@R: c(
    person("Nan", "Xiao", email = "me@nanx.me", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-0250-5673")),
    person("Qing-Song", "Xu", email = "qsxu@csu.edu.cn", role = c("aut")),
    person("Miao-Zhu", "Li", email = "miaozhu.li@duke.edu", role = c("aut")),
    person("Frank", "Harrell", email = "f.harrell@vanderbilt.edu", role = c("ctb"), comment = "rms author"),
    person("Sergej", "Potapov", role = c("ctb"), comment = "survAUC author"),
    person("Werner", "Adler", role = c("ctb"), comment = "survAUC author"),
    person("Matthias", "Schmid", role = c("ctb"), comment = "survAUC author")
    )
Description: Creates nomogram visualizations for penalized Cox regression
    models, with the support of reproducible survival model building,
    validation, calibration, and comparison for high-dimensional data.
License: GPL (>= 3)
LazyData: TRUE
URL: https://nanx.me/hdnom/, https://github.com/nanxstats/hdnom
BugReports: https://github.com/nanxstats/hdnom/issues
Depends: R (>= 3.5.0)
Imports: foreach, ggplot2, glmnet, gridExtra, ncvreg, penalized,
        survival
Suggests: doParallel, knitr, ragg, rmarkdown
VignetteBuilder: knitr
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-07-28 05:49:21 UTC; nanx
Author: Nan Xiao [aut, cre] (ORCID: <https://orcid.org/0000-0002-0250-5673>),
  Qing-Song Xu [aut],
  Miao-Zhu Li [aut],
  Frank Harrell [ctb] (rms author),
  Sergej Potapov [ctb] (survAUC author),
  Werner Adler [ctb] (survAUC author),
  Matthias Schmid [ctb] (survAUC author)
Maintainer: Nan Xiao <me@nanx.me>
Repository: CRAN
Date/Publication: 2026-07-28 06:50:02 UTC
Built: R 4.6.1; aarch64-apple-darwin23; 2026-07-28 09:30:32 UTC; unix
Archs: hdnom.so.dSYM
