# Generated by roxygen2: do not edit by hand

S3method(plot,hdnom.calibrate)
S3method(plot,hdnom.calibrate.external)
S3method(plot,hdnom.compare.calibrate)
S3method(plot,hdnom.compare.validate)
S3method(plot,hdnom.nomogram)
S3method(plot,hdnom.validate)
S3method(plot,hdnom.validate.external)
S3method(plot,nomogram.raw)
S3method(predict,hdnom.model)
S3method(print,hdnom.calibrate)
S3method(print,hdnom.calibrate.external)
S3method(print,hdnom.compare.calibrate)
S3method(print,hdnom.compare.validate)
S3method(print,hdnom.model)
S3method(print,hdnom.nomogram)
S3method(print,hdnom.validate)
S3method(print,hdnom.validate.external)
S3method(print,nomogram.raw)
S3method(summary,hdnom.calibrate)
S3method(summary,hdnom.calibrate.external)
S3method(summary,hdnom.compare.calibrate)
S3method(summary,hdnom.compare.validate)
S3method(summary,hdnom.validate)
S3method(summary,hdnom.validate.external)
export(as_nomogram)
export(calibrate)
export(calibrate_external)
export(compare_by_calibrate)
export(compare_by_validate)
export(fit_aenet)
export(fit_alasso)
export(fit_enet)
export(fit_flasso)
export(fit_lasso)
export(fit_mcp)
export(fit_mnet)
export(fit_scad)
export(fit_snet)
export(glmnet_basesurv)
export(glmnet_survcurve)
export(infer_variable_type)
export(kmplot)
export(logrank_test)
export(ncvreg_basesurv)
export(ncvreg_survcurve)
export(penalized_basesurv)
export(penalized_survcurve)
export(theme_hdnom)
export(validate)
export(validate_external)
importFrom(foreach,"%:%")
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,annotate)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_blank)
importFrom(ggplot2,geom_errorbar)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,geom_step)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_colour_manual)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,unit)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(glmnet,cv.glmnet)
importFrom(glmnet,glmnet)
importFrom(graphics,axis)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,segments)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(gridExtra,arrangeGrob)
importFrom(gridExtra,grid.arrange)
importFrom(ncvreg,cv.ncvsurv)
importFrom(ncvreg,ncvsurv)
importFrom(penalized,cvl)
importFrom(penalized,penalized)
importFrom(penalized,survival)
importFrom(stats,approx)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,formula)
importFrom(stats,median)
importFrom(stats,model.extract)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,pchisq)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,terms)
importFrom(survival,Surv)
importFrom(survival,survdiff)
importFrom(survival,survfit)
importMethodsFrom(penalized,predict)
useDynLib(hdnom,Cham_Diao)
useDynLib(hdnom,auc_SZ)
useDynLib(hdnom,auc_uno)
